import contextlib
import logging
import os
import sqlite3
from urllib.request import pathname2url

import requests
from flask import g, current_app

from . import consul


def run_query(query, args=()):
    with contextlib.closing(__get_db__()) as conn:  # auto-closes
        with conn:  # auto-commits
            with contextlib.closing(conn.cursor()) as cursor:  # auto-closes
                result = cursor.execute(query, args)
                return [dict(ix) for ix in result.fetchall()]


def __get_db__(mode="rw"):
    db_uri = 'file:{}?mode={}'.format(pathname2url(os.environ['DB_PATH']), mode)
    db = sqlite3.connect(
                db_uri,
                uri=True,
                detect_types=sqlite3.PARSE_DECLTYPES,
                isolation_level=None
            )
    db.row_factory = sqlite3.Row

    return db


def close_db(e=None):
    db = g.pop('db', None)

    if db is not None:
        db.close()


def init_db(app):
    try:
        __get_db__()  # Opening the DB as RW. This raises an exception if the file doesn't exist
    except sqlite3.OperationalError:
        __init_db__(app)


def __init_db__(app):
    logging.info("Initializing database...")
    db = __get_db__("rwc")

    with app.app_context():
        with current_app.open_resource('storage/schema.sql') as f:
            db.executescript(f.read().decode('utf8'))

    services = consul.find_services("appartments")
    if len(services) > 0:
        svc = services[0]
        appartments = requests.get(f"http://{svc['address']}:{svc['port']}/appartments").json()

        cursor = db.cursor()
        for entry in appartments["appartments"]:
            cursor.execute("INSERT INTO appartments VALUES (?, ?)", (entry["id"], entry["name"]))

        cursor.close()
        db.close()
        logging.info("Database initialized")
    else:
        logging.error("Cannot find an available Appartments instance")
        os.exit(1)
